//////////////////////////////////////////////////////////////////////////////////////
// Classe ModelMeteo: Correspon al Model del patro MVC
// Gestiona les dades dels arxius de configuracio
// Gestiona les dades provinents del canal RSS
/////////////////////////////////////////////////////////////////////////////////////
package widgets.MeteoclimaticRSS
{
	import com.esri.ags.Graphic;
	import com.esri.ags.Map;
	import com.esri.ags.geometry.Geometry;
	import com.esri.ags.geometry.MapPoint;
	import com.esri.ags.geometry.Polygon;
	import com.esri.ags.geometry.Polyline;
	import com.esri.ags.layers.GraphicsLayer;
	import com.esri.ags.symbols.PictureMarkerSymbol;
	import com.esri.ags.symbols.Symbol;
	import com.esri.ags.utils.WebMercatorUtil;
	import com.esri.ags.webmap.PopUpRenderer;
	import com.esri.ags.webmap.supportClasses.PopUpInfo;
	import com.esri.ags.webmap.supportClasses.PopUpMediaInfo;
	import com.esri.viewer.utils.GeoRSSUtil;
	import com.esri.viewer.utils.Namespaces;
	
	import flash.events.*;
	
	import mx.collections.ArrayCollection;
	import mx.collections.ArrayList;
	import mx.core.UIComponent;
	import mx.messaging.AbstractConsumer;
	
	
	
	public class ModelMeteo extends EventDispatcher
	{
		private static const ATOM:Namespace = Namespaces.ATOM_NS;
		
		protected var aComunitats:Array;
		protected var chosenComunitat:uint;
		protected var chosenProvincia:uint;
		protected var configuracio:Configuracio;
		protected var selectedVariable:String;
		protected var actualQuerySource:String;
		protected var estacionsFeeds:ArrayCollection;
		protected var showMessages:String;
			
		private var resultMarkerSymbol:Symbol;
		
		//
		// Constructor
		// carrega arxiu d configuració
		//
		public function ModelMeteo(configXML:XML)
		{
			var provincies:XMLList;
			provincies=configXML..provincia;
						
			var comunitats:XMLList;
			comunitats=configXML..comunitat;
			this.aComunitats = [];
			for (var i:int=0;i < comunitats.length();i++)
			{
				var comunitat:Comunitats= new Comunitats(i,comunitats[i].name,comunitats[i].url,provincies);
				aComunitats.push(comunitat);
			}
						
			this.chosenComunitat = 0;
			this.chosenProvincia = 0;
			
			configuracio=new Configuracio(configXML);
		}
		
		/////////////////////////////////////////
		// Metodes getvariables
		////////////////////////////////////////
		
		public function getComunitatsList():Array
		{
			return aComunitats;
		}
		public function getNomsComunitatsList():Array
		{
			var noms:Array=[];
			for (var i:int = 0; i < aComunitats.length; i++)
			{
				var comunitat:Comunitats = aComunitats[i];
				noms[i]=comunitat.getNom();
			}
			return noms;
		}
		
		public function getComunitat():uint
		{
			return this.chosenComunitat;
		}
		public function getObjComunitat():Comunitats
		{
			var comunitat:Array=this.getComunitatsList();
			return comunitat[this.getComunitat()];
		}
		
		
		public function getLengthComunitats():int
		{
			return this.aComunitats.length;
		}
				
		public function getProvincia():uint
		{
			return this.chosenProvincia;
		}
				
		public function getConfiguracio():Configuracio
		{
			return this.configuracio;
		}
		public function getNomComunitat():String
		{
			var comunitat:Comunitats = aComunitats[this.chosenComunitat];
			return comunitat.getNom();
		}
		public function getURLComunitat():String
		{
			var comunitat:Comunitats = aComunitats[this.chosenComunitat];
			return comunitat.getURLMeteo();
		}
		
		public function getActualQuerySource():String
		{
			return this.actualQuerySource;
		}
		
		public function getSelectedVariable():String
		{
			return this.selectedVariable;
		}
		public function getEstacionsFeeds():ArrayCollection
		{
			return this.estacionsFeeds;
		}
		public function getShowMessages():String
		{
			return this.showMessages;
		}
		
		/////////////////////////////////////////////
		// Metodes setvariables
		///////////////////////////////////////////////
		
		public function setComunitat(index:uint):void
		{
			this.chosenComunitat = index;
			setActualQuerySource(this.getURLComunitat());
			this.update( );
		}
		public function setShowMessages(Message:String):void
		{
			this.showMessages = Message;
			this.newMessage();
		}
		public function setEstacionsFeeds(pEstacionsFeeds:ArrayCollection):void
		{
			this.estacionsFeeds=pEstacionsFeeds;
		}
				
		public function setProvincia(index:uint):void
		{
			this.chosenProvincia = index;
			setActualQuerySource(Comunitats(this.aComunitats[this.chosenComunitat]).getURLProvincia(index));
		}
				
		
		public function setActualQuerySource(pActualQuerySource:String):void
		{
			this.actualQuerySource = pActualQuerySource;
		}
		
		public function setSelectedVariable(pSelectedVariable:String):void
		{
			this.selectedVariable = pSelectedVariable;
		}
		
		public function setConfiguracio(pConfiguracio:Configuracio):void
		{
			this.configuracio = pConfiguracio;
		}
		
		////////////////////////////////////////////////////////////////////////////////////////////
		//
		// Metodes de Gestio de les dades rebudes del canal RSS
		//
		////////////////////////////////////////////////////////////////////////////////////////////
		public function createEstacioRSSFeed(ac:ArrayCollection):void
		{
			var result:ArrayCollection = new ArrayCollection();
			var resultMarkerSymbolURL:String;
			var rang:Rangos= this.getActualRangValue();
			
			for (var i:Number = 0; i < ac.length; i++)
			{
				var recxml:XML = XML(ac.getItemAt(i));
				
				var geometry:Geometry = GeoRSSUtil.toGeometry(recxml);
				
				if (this.getConfiguracio().getTitleField())
				{
					var title:String = recxml[this.getConfiguracio().getTitleField()];
				}
				var content:String = "";
				var textFeed:String = "";
				
				for each (var fieldXML:XML in this.getConfiguracio().getQueryFields())
				{
					var value:String = recxml[fieldXML.@name[0]];
					var ActualVal:Number = this.getActualVariableValue(value);
				
					textFeed = rang.getVariable()+ ": " + ActualVal + " " + rang.getUnitats();
					resultMarkerSymbolURL=ConfigMarkerSymbolURL(rang,ActualVal)					
					if (value)
					{
						content = "Detall de les dades: " + value + "\n";
					}
				}

				var link:String;
				if (this.getConfiguracio().getLinkField())
				{
					link = recxml[this.getConfiguracio().getLinkField()];
				}
		
					
				geometry = WebMercatorUtil.geographicToWebMercator(geometry);
		
				var graphic:Graphic = new Graphic(geometry);
				graphic.checkForMouseListeners = false;
		
				var estacioRSSFeed:EstacioRSSFeed = new EstacioRSSFeed;
				estacioRSSFeed.title = title;
				estacioRSSFeed.valorActual = textFeed;
				estacioRSSFeed.content = content;
				estacioRSSFeed.point = getGeomCenter(graphic);
				estacioRSSFeed.link = link;
				estacioRSSFeed.geometry = geometry;
				estacioRSSFeed.graphic = graphic;
				
				if (geometry)
				{
					resultMarkerSymbol = new PictureMarkerSymbol(resultMarkerSymbolURL, 12,12,0,0);
					graphic.symbol = estacioRSSFeed.symbol = resultMarkerSymbol;
				}
				graphic.attributes = estacioRSSFeed;
				result.addItem(estacioRSSFeed);
								
			}
			this.setEstacionsFeeds(result);
			this.RSSupdated();
		}
		
		//
		// calcula i retorna punt central del objecte graphic passat
		//
		private function getGeomCenter(graphic:Graphic):MapPoint
		{
			var point:MapPoint;
			var geometry:Geometry = graphic.geometry;
			
			if (geometry)
			{
				switch (geometry.type)
				{
					case Geometry.MAPPOINT:
					{
						point = geometry as MapPoint;
						break;
					}
					case Geometry.POLYLINE:
					{
						var pl:Polyline = geometry as Polyline;
						var pathCount:Number = pl.paths.length;
						var pathIndex:int = int((pathCount / 2) - 1);
						var midPath:Array = pl.paths[pathIndex];
						var ptCount:Number = midPath.length;
						var ptIndex:int = int((ptCount / 2) - 1);
						point = pl.getPoint(pathIndex, ptIndex);
						break;
					}
					case Geometry.POLYGON:
					{
						const poly:Polygon = geometry as Polygon;
						point = poly.extent.center;
						break;
					}
				}
			}
			return point;
		}
		
		//
		// Cerca valor dada seleccionada entre la informacio arribada pel canal RSS
		//
		private function getActualVariableValue(value:String):Number
		{
			var parts:Array = value.split("<b>");
			
			switch (this.getSelectedVariable())
			{
				case 'Temperatura':
				{
					parts = parts[1].toString().split("</b>");
					break;
				}
				case 'Humitat':
				{
					parts = parts[2].toString().split("</b>");
					break;
				}
				case 'Pressió':
				{
					parts = parts[3].toString().split("</b>");
					break;
				}
				case 'Vent':
				{
					parts = parts[4].toString().split("</b>");
					break;
				}
				case 'Pluja':
				{
					parts = parts[6].toString().split("</b>");
					break;
				}
			}
			return parseFloat(parts[0].toString().replace(",","."));
		}

		//
		// Retorna els rangs a treballar segons la variable meteorologica seleccionada
		//
		public function getActualRangValue():Rangos
		{
		
			var rangos:Array = this.configuracio.getRangos();
			
			switch (this.getSelectedVariable())
			{
				case 'Temperatura':
				{
					var rang:Rangos = rangos[0];
					break;
				}
				case 'Humitat':
				{
					rang = rangos[1];
					break;
				}
				case 'Pressió':
				{
					rang = rangos[2];
					break;
				}
				case 'Vent':
				{
					rang = rangos[3];
					break;
				}
				case 'Pluja':
				{
					rang = rangos[4];
					break;
				}
			}
			return rang;
		}
		
		//
		// Retorna arxiu png (amb cami) corresponent segons valor actual i rang on queda ubicat 
		//
		public function ConfigMarkerSymbolURL(rang:Rangos,ActualVal:Number):String
		{
			var result:String = this.configuracio.getURLRang(0);
		
			if ((ActualVal > rang.getRang1() && !rang.getEscala()) || (ActualVal < rang.getRang1() && rang.getEscala()))
			{
				result = this.configuracio.getURLRang(1);
			}
			if ((ActualVal > rang.getRang2() && !rang.getEscala()) || (ActualVal < rang.getRang2() && rang.getEscala()))
			{
				result = this.configuracio.getURLRang(2);
			}
			if ((ActualVal > rang.getRang3() && !rang.getEscala()) || (ActualVal < rang.getRang3() && rang.getEscala()))
			{
				result = this.configuracio.getURLRang(3);
			}
			if ((ActualVal > rang.getRang4() && !rang.getEscala()) || (ActualVal < rang.getRang4() && rang.getEscala()))
			{
				result = this.configuracio.getURLRang(4);
			}
			return result;
		}
		
		//
		// Torna array de dades segons format ATOM
		//
		public function parseAtom(x:XML):ArrayCollection
		{
			var result:ArrayCollection = new ArrayCollection();
			for each (var entryXML:XML in x.ATOM::entry)
			{
				result.addItem(entryXML);
			}
			return result;
		}
		
		//
		// Torna array de dades segons format RSS
		//
		public function parseRSS(x:XML):ArrayCollection
		{
			var result:ArrayCollection = new ArrayCollection();
			for each (var itemXML:XML in x.channel.item)
			{
				result.addItem(itemXML);
			}
			return result;
		}

		///////////////////////////////////////////////////////////
		// Metodes generadors d'events per informar a la Vista 
		//////////////////////////////////////////////////////////
		protected function update( ):void
		{
			dispatchEvent(new Event(Event.CHANGE)); // dispatch event
		}
		protected function RSSupdated( ):void
		{
			dispatchEvent(new Event(Event.COMPLETE)); // dispatch event
		}
		protected function newMessage( ):void
		{
			dispatchEvent(new Event(Event.TEXT_INTERACTION_MODE_CHANGE)); // dispatch event
		}
	}
}